//ModbusRTU.cpp

#include <Arduino.h>
#include "config.h"
#include "ModbusRTU.h"
#include "maindat.h"

DAT_MRTU  dmrtu;


//инициализация
int16_t  sMRTU_INIT(void)
{
  memset(&dmrtu.rxbuf[0], 0, _szbuf);
  memset(&dmrtu.txbuf[0], 0, _szbuf);
  dmrtu.adrx = 0;
  dmrtu.adtx = 0;
  return 0;
}

//контрольная сумма
uint16_t sMRTU_CRC(uint8_t* data, uint16_t len)
{
  uint16_t res_CRC = 0xFFFF;
  uint16_t count = 0;
  uint8_t count_crc;
  uint8_t dt;
while(count < len)
{
  //
  count_crc = 0;
  dt = (uint8_t)(data[count]);
  res_CRC ^= (uint16_t)(dt);
  //
    while(count_crc < 8)
    {
      if((res_CRC & 0x0001) < 1)
      {
        res_CRC = (res_CRC >> 1) & 0x7FFF;
      }
      else
      {
        res_CRC = (res_CRC >> 1) & 0x7FFF;
        res_CRC ^= 0xA001;
      };
      count_crc++;
    };
    count++;
  }
  return (res_CRC);
}


/*
****************************
CMD6 - запись в один регистр
****************************
*/
int16_t	MRTU_CMD6(void)
{
	uint16_t  csize;
	uint16_t  crc; 		//подсчитанная контрольная сумма
	uint16_t  incrc;        //принятая контрольная сумма
	uint16_t  sreg  = 0;  	//стартовый регистр
	uint16_t  datreg = 0;
	csize  = 6;
      	sreg  |=(uint16_t) (* (uint8_t*)(&dmrtu.rxbuf[_ADDR_06_ADDR_REG_H])) << 8;
      	sreg  |= (uint16_t) (*(uint8_t*)(&dmrtu.rxbuf[_ADDR_06_ADDR_REG_H + 1]));
      	datreg  |= (uint16_t) (* (uint8_t*)(&dmrtu.rxbuf[_ADDR_06_DAT_H])) << 8;
      	datreg  |= (uint16_t) (*(uint8_t*)(&dmrtu.rxbuf[_ADDR_06_DAT_H + 1]));
      	incrc  = (*(uint16_t*)(&dmrtu.rxbuf[_ADDR_06_INCRC]));
      	crc = sMRTU_CRC(&dmrtu.rxbuf[0], csize);
      	if(crc == incrc)
      	{
        	/*
        	****************************
        	проверка границ данных
        	****************************
        	*/
        	//ComMotion
        	if((sreg >= _ComMotion_RMS) && (sreg < (_ComMotion_RMS + _ComMotion_SzRM)))
        	{
           		dmotor[sreg - _ComMotion_RMS] = (int16_t)(datreg);
           		memcpy(&dmrtu.txbuf[0], &dmrtu.rxbuf[0], 8);
           		Serial.write(&dmrtu.txbuf[0], 8);
           		Serial.flush();
        	}
		//Multiservo
		else if((sreg >= _Multiservo_RSS1) && (sreg < (_Multiservo_RSS1 + _Multiservo_SzRS)))
		{
			dservo[sreg - _Multiservo_RSS1] = (int16_t)(datreg);
			memcpy(&dmrtu.txbuf[0], &dmrtu.rxbuf[0], 8);
			Serial.write(&dmrtu.txbuf[0], 8);
			Serial.flush();
		}
		// ILLEGAL DATA ADDRESS - вышли за предел данных
		else
		{
			(*(uint8_t*)(&dmrtu.txbuf[0]))  =  (*(uint8_t*)(&dmrtu.rxbuf[0]));
			(*(uint8_t*)(&dmrtu.txbuf[1]))  =  (*(uint8_t*)(&dmrtu.rxbuf[_ADDR_MCMD])) | _CMD_ERR_MODE;
			(*(uint8_t*)(&dmrtu.txbuf[2])) =  _ILLEGAL_DATA_ADDRESS;
			crc = sMRTU_CRC(&dmrtu.txbuf[0], 3);
			(*(uint16_t*)(&dmrtu.txbuf[3])) = crc;
			Serial.write(&dmrtu.txbuf[0], 5);
			Serial.flush();
		}
        //
	}
	return 0;
}

/*
****************************
неопределённая команда
****************************
*/
int16_t	MRTU_CMDunknown(void)
{
	uint16_t  crc; //подсчитанная контрольная сумма
	(*(uint8_t*)(&dmrtu.txbuf[0]))  =  (*(uint8_t*)(&dmrtu.rxbuf[0]));
       	(*(uint8_t*)(&dmrtu.txbuf[1]))  =  (*(uint8_t*)(&dmrtu.rxbuf[1])) | _CMD_ERR_MODE;
       	(*(uint8_t*)(&dmrtu.txbuf[2])) =  _ILLEGAL_FUNCTION;
       	crc = sMRTU_CRC(&dmrtu.txbuf[0], 3);
       	(*(uint16_t*)(&dmrtu.txbuf[3])) = crc;
       	Serial.write(&dmrtu.txbuf[0], 5);
       	Serial.flush();
	return 0;
}

/*
****************************
приём команды и обработка
****************************
*/
int16_t  MRTU_rWORK(void)
{
	int rcvsz = Serial.available();	//принятый размер
	uint8_t  resb;
	int rcnt = 0, rccnt = 0;
	uint8_t wcnt = 0;
	//нет пакета для обработки
	if(rcvsz < 8)
		return 0;
	wcnt = Serial.readBytes((char*)(&dmrtu.rxbuf[0]), _szbuf);
	//не наш адрес
  	if(dmrtu.rxbuf[_ADDR_MADDR] != _MRTU_Addr)
		return 0;
	//определение команд
	switch(dmrtu.rxbuf[_ADDR_MCMD])
	{
		case 6:
			MRTU_CMD6();
			break;
		default:
			MRTU_CMDunknown();
			break;
	}
	//обработка принятых данных
	//return sMRTU_CMD(&dmrtu.rxbuf[0], wcnt, &dmrtu.txbuf[0]);
}


